@@ -27,40 +27,54 @@ using namespace PGSTD;
 using namespace pqxx::internal::pq;
 
 
-pqxx::result &pqxx::result::operator=(const pqxx::result &Other) throw ()
+bool pqxx::result::operator==(const result &rhs) const throw ()
 {
-  if (&Other != this)
-  {
-    LoseRef();
-    MakeRef(Other);
-  }
-
-  return *this;
+  if (&rhs == this) return true;
+  const size_type s(size());
+  if (rhs.size() != s) return false;
+  for (size_type i=0; i<s; ++i)
+    if ((*this)[i] != rhs[i]) return false;
+  return true;
 }
 
 
-pqxx::result &pqxx::result::operator=(internal::pq::PGresult *Other) throw ()
+bool pqxx::result::tuple::operator==(const tuple &rhs) const throw ()
 {
-  if (Other != m_Result)
-  {
-    LoseRef();
-    MakeRef(Other);
-  }
-  return *this;
+  if (&rhs == this) return true;
+  const size_type s(size());
+  if (rhs.size() != s) return false;
+  // TODO: Depends on how null is handled!
+  for (size_type i=0; i<s; ++i) if ((*this)[i] != rhs[i]) return false;
+  return true;
 }
 
 
-void pqxx::result::swap(pqxx::result &other) throw ()
+void pqxx::result::tuple::swap(tuple &rhs) throw ()
 {
-  const result *const l=m_l, *const r=m_r;
-  internal::pq::PGresult *p(m_Result);
+  const result *const h(m_Home);
+  const result::size_type i(m_Index);
+  m_Home = rhs.m_Home;
+  m_Index = rhs.m_Index;
+  rhs.m_Home = h;
+  rhs.m_Index = i;
+}
 
-  m_l = other.m_l;
-  m_r = other.m_r;
-  m_Result = other.m_Result;
-  other.m_l = l;
-  other.m_r = r;
-  other.m_Result = p;
+
+bool pqxx::result::field::operator==(const field &rhs) const
+{
+  if (is_null() != rhs.is_null()) return false;
+  // TODO: Verify null handling decision
+  const size_type s = size();
+  if (s != rhs.size()) return false;
+  const char *const l(c_str()), *const r(rhs.c_str());
+  for (size_type i = 0; i < s; ++i) if (l[i] != r[i]) return false;
+  return true;
+}
+
+
+void pqxx::result::swap(result &rhs) throw ()
+{
+  super::swap(rhs);
 }
 
 
@@ -90,12 +104,12 @@ void pqxx::result::CheckStatus(const char Query[]) const
 
 string pqxx::result::StatusError() const
 {
-  if (!m_Result)
+  if (!c_ptr())
     throw runtime_error("No result");
 
   string Err;
 
-  switch (PQresultStatus(m_Result))
+  switch (PQresultStatus(c_ptr()))
   {
   case PGRES_EMPTY_QUERY: // The string sent to the backend was empty.
   case PGRES_COMMAND_OK: // Successful completion of a command returning no data
@@ -109,48 +123,21 @@ string pqxx::result::StatusError() const
   case PGRES_BAD_RESPONSE: // The server's response was not understood
   case PGRES_NONFATAL_ERROR:
   case PGRES_FATAL_ERROR:
-    Err = PQresultErrorMessage(m_Result);
+    Err = PQresultErrorMessage(c_ptr());
     break;
 
   default:
     throw logic_error("libpqxx internal error: "
 		      "pqxx::result: Unrecognized response code " +
-		      to_string(int(PQresultStatus(m_Result))));
+		      to_string(int(PQresultStatus(c_ptr()))));
   }
   return Err;
 }
 
 
-void pqxx::result::MakeRef(internal::pq::PGresult *Other) throw ()
-{
-  m_Result = Other;
-}
-
-
-void pqxx::result::MakeRef(const pqxx::result &Other) throw ()
-{
-  m_l = &Other;
-  m_r = Other.m_r;
-
-  m_l->m_r = m_r->m_l = this;
-  m_Result = Other.m_Result;
-}
-
-
-void pqxx::result::LoseRef() throw ()
-{
-  if ((m_l == this) && m_Result) PQclear(m_Result);
-  m_Result = 0;
-  m_l->m_r = m_r;
-  m_r->m_l = m_l;
-  m_l = m_r = this;
-}
-
-
-
 pqxx::result::size_type pqxx::result::affected_rows() const
 {
-  const char *const RowsStr = PQcmdTuples(m_Result);
+  const char *const RowsStr = PQcmdTuples(c_ptr());
   return RowsStr[0] ? atoi(RowsStr) : 0;
 }
 
@@ -158,31 +145,59 @@ pqxx::result::size_type pqxx::result::affected_rows() const
 const char *pqxx::result::GetValue(pqxx::result::size_type Row, 
 		                 pqxx::result::tuple::size_type Col) const
 {
-  return PQgetvalue(m_Result, Row, Col);
+  return PQgetvalue(c_ptr(), Row, Col);
 }
 
 
 bool pqxx::result::GetIsNull(pqxx::result::size_type Row,
 		           pqxx::result::tuple::size_type Col) const
 {
-  return PQgetisnull(m_Result, Row, Col) != 0;
+  return PQgetisnull(c_ptr(), Row, Col) != 0;
 }
 
 pqxx::result::field::size_type 
 pqxx::result::GetLength(pqxx::result::size_type Row,
                         pqxx::result::tuple::size_type Col) const
 {
-  return PQgetlength(m_Result, Row, Col);
+  return PQgetlength(c_ptr(), Row, Col);
 }
 
 
+pqxx::oid pqxx::result::column_type(tuple::size_type ColNum) const
+{
+  const oid T = PQftype(c_ptr(), ColNum);
+  if (T == oid_none)
+    throw PGSTD::invalid_argument(
+	"Attempt to retrieve type of nonexistant column " +
+	to_string(ColNum) + " of query result");
+  return T;
+}
+
+
+#ifdef PQXX_HAVE_PQFTABLE
+pqxx::oid pqxx::result::column_table(tuple::size_type ColNum) const
+{
+  const oid T = PQftable(c_ptr(), ColNum);
+
+  /* If we get oid_none, it may be because the column is computed, or because we
+   * got an invalid row number.
+   */
+  if (T == oid_none && ColNum >= columns())
+    throw PGSTD::invalid_argument("Attempt to retrieve table ID for column " +
+	to_string(ColNum) + " out of " + to_string(columns()));
+
+  return T;
+}
+#endif
+
+
 int pqxx::result::errorposition() const throw ()
 {
   int pos = -1;
 #if defined(PQXX_HAVE_PQRESULTERRORFIELD)
-  if (m_Result)
+  if (c_ptr())
   {
-    const char *p = PQresultErrorField(m_Result, PG_DIAG_STATEMENT_POSITION);
+    const char *p = PQresultErrorField(c_ptr(), PG_DIAG_STATEMENT_POSITION);
     if (p) from_string(p, pos);
   }
 #endif // PQXX_HAVE_PQRESULTERRORFIELD
@@ -221,7 +236,7 @@ pqxx::result::tuple::at(pqxx::result::tuple::size_type i) const throw (out_of_ra
 const char *
 pqxx::result::column_name(pqxx::result::tuple::size_type Number) const
 {
-  const char *const N = PQfname(m_Result, Number);
+  const char *const N = PQfname(c_ptr(), Number);
   if (!N) 
     throw out_of_range("Invalid column number: " + to_string(Number));
 
@@ -232,7 +247,7 @@ pqxx::result::column_name(pqxx::result::tuple::size_type Number) const
 pqxx::result::tuple::size_type
 pqxx::result::column_number(const char ColName[]) const
 {
-  const int N = PQfnumber(m_Result, ColName);
+  const int N = PQfnumber(c_ptr(), ColName);
   if (N == -1)
     throw invalid_argument("Unknown column name: '" + string(ColName) + "'");
 
@@ -242,7 +257,8 @@ pqxx::result::column_number(const char ColName[]) const
 
 // const_iterator
 
-pqxx::result::const_iterator pqxx::result::const_iterator::operator++(int)
+pqxx::result::const_iterator
+pqxx::result::const_iterator::operator++(int)
 {
   const_iterator old(*this);
   m_Index++;
@@ -250,7 +266,8 @@ pqxx::result::const_iterator pqxx::result::const_iterator::operator++(int)
 }
 
 
-pqxx::result::const_iterator pqxx::result::const_iterator::operator--(int)
+pqxx::result::const_iterator
+pqxx::result::const_iterator::operator--(int)
 {
   const_iterator old(*this);
   m_Index--;
@@ -278,3 +295,39 @@ pqxx::result::const_fielditerator::operator--(int)
   return old;
 }
 
+
+pqxx::result::const_reverse_iterator
+pqxx::result::const_reverse_iterator::operator++(int)
+{
+  const_reverse_iterator tmp(*this);
+  iterator_type::operator--();
+  return tmp;
+}
+
+
+pqxx::result::const_reverse_iterator
+pqxx::result::const_reverse_iterator::operator--(int)
+{
+  const_reverse_iterator tmp(*this);
+  iterator_type::operator++();
+  return tmp;
+}
+
+pqxx::result::const_reverse_fielditerator
+pqxx::result::const_reverse_fielditerator::operator++(int)
+{
+  const_reverse_fielditerator tmp(*this);
+  iterator_type::operator--();
+  return tmp;
+}
+
+
+pqxx::result::const_reverse_fielditerator
+pqxx::result::const_reverse_fielditerator::operator--(int)
+{
+  const_reverse_fielditerator tmp(*this);
+  iterator_type::operator++();
+  return tmp;
+}
+
+
